
require 'yaml'

# ========================================================================
# ApplicationController: アプリケーションクラス.
class ApplicationController < ActionController::Base

  # Prevent CSRF attacks by raising an exception.
  # For APIs, you may want to use :null_session instead.
  protect_from_forgery with: :exception

  # locale setting
  # ref. http://ruby-rails.hatenadiary.com/entry/20150226/1424937175
  before_action :set_locale
  def set_locale
    I18n.locale = params[:locale] || I18n.default_locale
  end
  def default_url_options(options = {})
    {locale: I18n.locale}.merge options
  end

  # viewから呼び出すメソッド定義 (helper_method).
  helper_method :current_user
  helper_method :maintenance

  private
  def current_user
    @current_user ||= User.find(session[:user_id]) if session[:user_id]
  end

  # IPアドレス制限の判断.
  private
  def accepted?(ip)
    # リスト取得(設定ファイルから).
    whitelist = SHIBCERT_CONFIG[Rails.env]['access_white_ip']
    blacklist = SHIBCERT_CONFIG[Rails.env]['access_black_ip']
    # ホワイトリストチェック.
    access = false
    if !whitelist.blank?
      # ホワイトリストあり.
      whites = whitelist.split(",")
      if !whites.blank?
        # 確認
        whites.each do |wip|
          ipaddr = IPAddr.new(wip.strip)
          if ipaddr.include?(ip)
            access = true       # アクセス許可.
            break
          end
        end
      end
    else
      # ホワイトリストが無い場合は全てアクセス許可.
      access = true
    end
    # ブラックリストチェック.
    if !blacklist.blank?
      # ブラックリストあり
      blacks = blacklist.split(",")
      if !blacks.blank?
        # 確認
        blacks.each do |bip|
          ipaddr = IPAddr.new(bip.strip)
          if ipaddr.include?(ip)
            access = false      # アクセス拒否.
            break
          end
        end
      end
    end
    return access
  end

  # IPアドレス制限と拒否時のリダイレクト.
  private
  def block_foreign_hosts
    redirect = SHIBCERT_CONFIG[Rails.env]['access_reject_to']
    redirect = "/403.html" if redirect.blank?     # デフォルトは 403.html
    return false if accepted?(request.remote_ip)
    redirect_to redirect
  end

  # メンテナンス中の確認(ファイルの有無で判定).
  private
  def maintenance?
    return File.exist?("config/maintenance.yml")
  end

  # メンテナンス中文字列の取得.
  private
  def maintenance_mesg
    if !maintenance?
      return nil
    end
    mesg = nil
    yml = YAML.load_file("config/maintenance.yml")
    if !yml.blank?
      mesg = yml[params[:locale]]
    end
    if mesg.blank?
      yml_def = YAML.load_file("config/maintenance.yml.default")
      if !yml_def.blank?
        mesg = yml_def[params[:locale]]
      end
    end
    if mesg.blank?
      mesg = "maintenance now."
    end
    return mesg
  end

end
# ========================================================================


